/* 
 * File:   mute_revC.c
 * Author: Mark
 *
 * Created on November 19, 2015, 3:26 PM
 */

#define BUILD_NUMBER    (11)

// Designed for PIC 12F1571

#include <xc.h>

// CONFIG1
#pragma config FOSC = INTOSC    //  (INTOSC oscillator; I/O function on CLKIN pin)
#pragma config WDTE = ON        // Watchdog Timer Enable (WDT enabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOREN = OFF    // Low Power Brown-out Reset enable bit (LPBOR is disabled)
#pragma config LVP = ON         // Low-Voltage Programming Enable (Low-voltage programming enabled)


#define _XTAL_FREQ (500000)     // Tell the compiler you're running at 500 kHz

#define MIN_SLEEP_20MS (3)      // Minimum stay-unmuted time from AC lost, till muted  (increments of 20 msec)
#define MAX_SLEEP_20MS (200)    // Maximum stay-unmuted time from AC lost, till muted
#define MIN_WAKE_20MS  (100)    // Minimum stay-muted time from AC good, till unmuted
#define MAX_WAKE_20MS  (2000)   // Maximum stay-muted time from AC good, till unmuted

// AHA!! note to self: Do Not Trust The Preprocessor!!
//                     I wrote the slopes and intercepts as expressions in the
//                     preprocessor, and positive decimal numbers > 128 got
//                     turned into signed integers < 0 by the preprocessor
#define SLEEP_INTERCEPT (3)     // equals MIN_SLEEP_20MS
#define SLEEP_SLOPE     (198)   // (MAX-MIN)*(256/255)
#define WAKE_INTERCEPT  (100)   // equals MIN_WAKE_20MS
#define WAKE_SLOPE      (1907)  // (MAX-MIN)*(256/255)

#define START_STATE     (1)
#define TURNON_STATE    (2)
#define PLAY_MUSIC_STATE    (3)
#define SHUTDOWN_STATE  (4)

#define AC_PRESENT_MASK (0b00100000)        // ac_present comes in on RA5
#define AC_PRESENT_SHIFT (5)


#include <stdio.h>
#include <stdlib.h>



// GLOBAL VARIABLES (so they don't have to be passed as parameters)
// GLOBAL VARIABLES (so they don't have to be passed as parameters)
unsigned    char    vwake, vsleep ;
unsigned    short   dly_wake_20ms, dly_sleep_20ms ;   // 16 bit numbers

    

// return the uppermost 8 bits of the 10 bit ADC result
unsigned char ADC_read(unsigned char ADchannel) {

                                    // Since we are using INTOSC@0.5MHz,
                                    // Microchip requires ADClock = Fosc/2
    ADCON1 = 0b00000000 ;           // ADFM = 0,  Clock=Fosc/2, Vref=VDD
    ADCON0 = ((ADchannel & 0x03) << 2) | 0b00000001 ;
                                    // ADON=1 (on) in bits 0,1  channel# in bits 2,3
                                    // Sample time = 2us for the amplifier settling
                                    // plus 4us for worst case temperature (100C)
                                    // plus 12.5pF*(8K + 25K)*6.3 = 2.8us
    __delay_us(18) ;                // let the ADC do its sampling
                                    // now turn on the GO bit
    ADCON0 = ((ADchannel & 0x03) << 2) | 0b00000011 ;
    while(0 != (ADCON0 & 0b00000010)) {     // wait until DONE
        __delay_us(1) ;
    }
    return( ADRESH ) ;      // capture the uppermost 8b of the 10b ADC result
}


// convert ADC voltages into delays, in increments of 20 milliseconds
void    volts_to_20ms_intervals( void ) {
    
    unsigned    long    t1, t2, t3, t4 ;    // four 32-bit temporaries
    
    t1 = vwake ;
    t2 = WAKE_INTERCEPT ;
    t3 = WAKE_SLOPE ;
    t4 = t1 * t3 ;
    t4 = t2 + ((0x3fffffff & t4) >> 8) ;           // divide by 256
    dly_wake_20ms = t4 ;
    
    t1 = vsleep ;
    t2 = SLEEP_INTERCEPT ;
    t3 = SLEEP_SLOPE ;
    t4 = t1 * t3 ;
    t4 = t2 + ((0x3fffffff & t4) >> 8) ;           // divide by 256
    dly_sleep_20ms = t4 ;
  
}


// variable delay = (ARGUMENT * 20 milliseconds)
void    my_20ms_delay( unsigned short t ) {
    unsigned short j;
    j = t;
    while(j > 0) {
        __delay_ms(20) ;
        j--;
    }
}

    

int main(int argc, char** argv) {    

    unsigned    char    cur_state, prev_state ;
    unsigned    char    ac_present, mute_bar ;
    
    
    // Get the build number into the HEX file
    cur_state = BUILD_NUMBER ;
    
    
// The program is a Finite State Machine with four states.  This is the first state    

state_start:
    prev_state = cur_state ;        // these do nothing at all.  Good for printf's I suppose
    cur_state = START_STATE ;
    mute_bar = 0 ;                  // MUTED!!  When we just begin, mute the output

    TRISA =  0b00111100 ;           // Inputs on PORTA bits 5, 4, 3, 2
    ANSELA = 0b00010100 ;           // A-to-D on PORTA bits 4, 2
    PORTA =  (0x01 & mute_bar) ;    // Write zeroes to all other outputs, mute_bar to RA0
    
    WDTCON = 0b00100011 ;           // Watchdog timer on; times out at 128 seconds
    
    my_20ms_delay(12) ;             // Wait 240 milliseconds before reading ADCs
                                    //      because we don't really know how soon
                                    //      the Power On Reset goes away and lets
                                    //      CPU start executing.  It might be as
                                    //      low as VDD = 3.0V ???
                                    //      Give ourselves 14 cycles of the 60Hz
                                    //      AC sinewave, to pump VDD up some more
    
    vsleep = ADC_read(2) ;          // Read ADC channel 2 (pin RA2)
    __delay_us(50) ;                // Put wakeup ADC read last, giving its input RC extra time to settle
    vwake = ADC_read(3) ;           // Read ADC channel 3 (pin RA4)
    
    CLRWDT() ;                       // clear the watchdog timer
    
    ac_present = PORTA ;            // read the digital input
    ac_present = (ac_present & AC_PRESENT_MASK) >> AC_PRESENT_SHIFT ;
    
    if(0 == ac_present) goto state_start ;
    else goto state_turnon ;
    

// Enter this state when AC power becomes available.  Delay the "wake" time before un-muting  
state_turnon:
    prev_state = cur_state ;
    cur_state = TURNON_STATE ;    
    mute_bar = 0 ;                  // MUTED!!
    PORTA =  (0x01 & mute_bar) ;    // Write zeroes to all other outputs, mute_bar to RA0
    
    volts_to_20ms_intervals() ;
    my_20ms_delay(dly_wake_20ms) ;
    
    // Double check to see if AC power disappeared-- it would be bad to unmute then!
    ac_present = PORTA ;
    ac_present = (ac_present & AC_PRESENT_MASK) >> AC_PRESENT_SHIFT ;
    
    CLRWDT() ;                       // clear the watchdog timer
    
    if(0 == ac_present) goto state_start ;
    else goto state_play_music ;
    

// Enter this state when AC power has been good for a while and we no longer need to be muted    
state_play_music:
    prev_state = cur_state ;
    cur_state = PLAY_MUSIC_STATE ;    
    mute_bar = 1 ;                  // cease muting -- play music!
    PORTA =  (0x01 & mute_bar) ;    // Write zeroes to all other outputs, mute_bar to RA0
    
    // We may as well read the ADCs again, we've got nothing else to do
    // for hours, while the user plays music.  Besides, the ADC values are
    // good and settled by now.  Good and warmed up.
    
    vwake = ADC_read(3) ;           // Read ADC channel 3 (pin RA4)
    vsleep = ADC_read(2) ;          // Read ADC channel 2 (pin RA2)
    
    volts_to_20ms_intervals() ;     // might as well do some math while
                                    // we sit here waiting for the user to
                                    // turn off the AC mains power
    
    CLRWDT() ;                      // clear the watchdog timer
    
    ac_present = PORTA ;
    ac_present = (ac_present & AC_PRESENT_MASK) >> AC_PRESENT_SHIFT ;
    
    if(0 == ac_present) goto state_shutdown ;
    else goto state_play_music ;
    

// Enter this state at the instant when AC power disappears.
// Remain not-muted for a little while, hoping AC power will return.
// If it does not return then SHUT-DOWN and mute.    
state_shutdown:
    prev_state = cur_state ;
    cur_state = SHUTDOWN_STATE ;    
    mute_bar = 1 ;                  // not-muted for a few milliseconds more
    PORTA =  (0x01 & mute_bar) ;    // Write zeroes to all other outputs, mute_bar to RA0
    
    volts_to_20ms_intervals() ;
    my_20ms_delay(dly_sleep_20ms) ;
    
    ac_present = PORTA ;
    ac_present = (ac_present & AC_PRESENT_MASK) >> AC_PRESENT_SHIFT ;
    
    CLRWDT() ;                       // clear the watchdog timer   
    
    // Maybe the AC power was restored during that delay (??)
    //    if it was then keep playing music
    if(0 == ac_present) {
        mute_bar = 0 ;                  // MUTED!!
        PORTA =  (0x01 & mute_bar) ;    // Write zeroes to all other outputs, mute_bar to RA0
        goto state_start ;
    }
    else goto state_play_music ;


    return (EXIT_SUCCESS);
}
